<?php

use Phinx\Migration\AbstractMigration;
use Phinx\Db\Adapter\MysqlAdapter;

class AddCms extends AbstractMigration
{
    public function change()
    {
        $this
            ->table("cms_menu_position", ['id' => false, 'primary_key' => ["menu_position_id"], 'engine' => "InnoDB", 'encoding' => "utf8", 'collation' => "utf8_general_ci", 'comment' => ""])
            ->addColumn('menu_position_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'identity' => 'enable', 'signed' => false])
            ->addColumn('menu_position_nom', 'string', ['null' => false, 'default' => '', 'limit' => 255, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'menu_position_id'])
            ->create();

        $this
            ->table("cms_menu_statut", ['id' => false, 'primary_key' => ["menu_statut_id"], 'engine' => "InnoDB", 'encoding' => "utf8", 'collation' => "utf8_general_ci", 'comment' => ""])
            ->addColumn('menu_statut_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'identity' => 'enable', 'signed' => false])
            ->addColumn('menu_statut_nom', 'string', ['null' => false, 'default' => '', 'limit' => 255, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'menu_statut_id'])
            ->create();

        $this
            ->table("cms_menu", ['id' => false, 'primary_key' => ["cms_menu_id"], 'engine' => "InnoDB", 'encoding' => "utf8", 'collation' => "utf8_general_ci", 'comment' => ""])
            ->addColumn('cms_menu_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'identity' => 'enable', 'signed' => false])
            ->addColumn('menu_position_id', 'integer', ['null' => false, 'default' => '0', 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'cms_menu_id', 'signed' => false])
            ->addColumn('parent_id', 'integer', ['null' => true, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'menu_position_id', 'signed' => false])
            ->addColumn('menu_date_ajout', 'datetime', ['null' => false, 'default' => '0000-00-00 00:00:00', 'after' => 'parent_id'])
            ->addColumn('menu_date_modif', 'datetime', ['null' => false, 'default' => '0000-00-00 00:00:00', 'after' => 'menu_date_ajout'])
            ->addColumn('menu_rang', 'integer', ['null' => false, 'default' => '0', 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'menu_date_modif'])
            ->addColumn('menu_statut_id', 'integer', ['null' => false, 'default' => '0', 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'menu_rang', 'signed' => false])
            ->addForeignKey('menu_position_id', 'cms_menu_position', 'menu_position_id', array(
                'delete' => 'RESTRICT',
                'update' => 'CASCADE'
            ))
            ->addForeignKey('menu_statut_id', 'cms_menu_statut', 'menu_statut_id', array(
                'delete' => 'RESTRICT',
                'update' => 'CASCADE'
            ))
            ->addForeignKey('parent_id', 'cms_menu', 'cms_menu_id', array(
                'delete' => 'SET NULL',
                'update' => 'CASCADE'
            ))
            ->addIndex(['parent_id'], ['name' => "parent_id", 'unique' => false])
            ->addIndex(['menu_position_id'], ['name' => "menu_position_id", 'unique' => false])
            ->addIndex(['menu_statut_id'], ['name' => "menu_statut_id", 'unique' => false])
            ->addIndex(['menu_rang'], ['name' => "menu_rang", 'unique' => false])
            ->create();

        $this
            ->table("cms_menu_boutique", ['id' => false, 'primary_key' => ["cms_menu_boutique_id"], 'engine' => "InnoDB", 'encoding' => "utf8", 'collation' => "utf8_general_ci", 'comment' => ""])
            ->addColumn('cms_menu_boutique_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'identity' => 'enable', 'signed' => false])
            ->addColumn('cms_menu_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'cms_menu_boutique_id', 'signed' => false])
            ->addColumn('boutique_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'cms_menu_id', 'signed' => false])
            ->addColumn('cms_menu_boutique_statut_id', 'integer', ['null' => false, 'default' => '1', 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'boutique_id'])
            ->addForeignKey('cms_menu_id', 'cms_menu', 'cms_menu_id', array(
                'delete' => 'CASCADE',
                'update' => 'CASCADE'
            ))
            ->addForeignKey('boutique_id', 'boutique', 'boutique_id', array(
                'delete' => 'CASCADE',
                'update' => 'CASCADE'
            ))
            ->addIndex(['cms_menu_id', 'boutique_id'], ['name' => "cms_menu_id_2", 'unique' => true])
            ->addIndex(['cms_menu_id'], ['name' => "cms_menu_id", 'unique' => false])
            ->addIndex(['boutique_id'], ['name' => "boutique_id", 'unique' => false])
            ->addIndex(['cms_menu_boutique_statut_id'], ['name' => "cms_menu_boutique_statut_id", 'unique' => false])
            ->create();

        $this
            ->table("cms_menu_nom", ['id' => false, 'primary_key' => ["cms_menu_nom_id"], 'engine' => "InnoDB", 'encoding' => "utf8", 'collation' => "utf8_general_ci", 'comment' => ""])
            ->addColumn('cms_menu_nom_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'identity' => 'enable', 'signed' => false])
            ->addColumn('cms_menu_id', 'integer', ['null' => false, 'default' => '0', 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'cms_menu_nom_id', 'signed' => false])
            ->addColumn('langue_id', 'integer', ['null' => false, 'default' => '1', 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'cms_menu_id', 'signed' => false])
            ->addColumn('menu_nom_nom', 'string', ['null' => false, 'default' => '', 'limit' => 255, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'langue_id'])
            ->addColumn('menu_nom_url', 'string', ['null' => false, 'default' => '', 'limit' => 255, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'menu_nom_nom'])
            ->addForeignKey('langue_id', 'langue', 'langue_id', array(
                'delete' => 'CASCADE',
                'update' => 'CASCADE'
            ))
            ->addForeignKey('cms_menu_id', 'cms_menu', 'cms_menu_id', array(
                'delete' => 'CASCADE',
                'update' => 'CASCADE'
            ))
            ->addIndex(['langue_id', 'cms_menu_id'], ['name' => "langue_id_2", 'unique' => true])
            ->addIndex(['cms_menu_id'], ['name' => "cms_menu_id", 'unique' => false])
            ->addIndex(['langue_id'], ['name' => "langue_id", 'unique' => false])
            ->create();

        $this
            ->table("cms_page_statut", ['id' => false, 'primary_key' => ["page_statut_id"], 'engine' => "InnoDB", 'encoding' => "utf8", 'collation' => "utf8_general_ci", 'comment' => ""])
            ->addColumn('page_statut_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'identity' => 'enable', 'signed' => false])
            ->addColumn('page_statut_nom', 'string', ['null' => false, 'default' => '', 'limit' => 255, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'page_statut_id'])
            ->create();

        $this
            ->table("cms_page", ['id' => false, 'primary_key' => ["cms_page_id"], 'engine' => "InnoDB", 'encoding' => "utf8", 'collation' => "utf8_general_ci", 'comment' => ""])
            ->addColumn('cms_page_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'identity' => 'enable', 'signed' => false])
            ->addColumn('cms_menu_id', 'integer', ['null' => true, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'cms_page_id', 'signed' => false])
            ->addColumn('parent_id', 'integer', ['null' => true, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'cms_menu_id', 'signed' => false])
            ->addColumn('page_rang', 'integer', ['null' => false, 'default' => '0', 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'parent_id'])
            ->addColumn('page_date_ajout', 'datetime', ['null' => false, 'default' => '0000-00-00 00:00:00', 'after' => 'page_rang'])
            ->addColumn('page_date_modif', 'datetime', ['null' => false, 'default' => '0000-00-00 00:00:00', 'after' => 'page_date_ajout'])
            ->addColumn('page_statut_id', 'integer', ['null' => false, 'default' => '0', 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'page_date_modif', 'signed' => false])
            ->addColumn('cms_page_type', 'enum', ['null' => false, 'limit' => 5, 'values' => ['page', 'alias', 'block'], 'after' => 'page_statut_id'])
            ->addColumn('alias_id', 'integer', ['null' => true, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'cms_page_type', 'signed' => false])
            ->addForeignKey('cms_menu_id', 'cms_menu', 'cms_menu_id', array(
                'delete' => 'SET NULL',
                'update' => 'CASCADE'
            ))
            ->addForeignKey('parent_id', 'cms_page', 'cms_page_id', array(
                'delete' => 'CASCADE',
                'update' => 'CASCADE'
            ))
            ->addForeignKey('page_statut_id', 'cms_page_statut', 'page_statut_id', array(
                'delete' => 'RESTRICT',
                'update' => 'CASCADE'
            ))
            ->addForeignKey('alias_id', 'cms_page', 'cms_page_id', array(
                'delete' => 'SET NULL',
                'update' => 'CASCADE'
            ))
            ->addIndex(['cms_menu_id'], ['name' => "cms_menu_id", 'unique' => false])
            ->addIndex(['parent_id'], ['name' => "parent_id", 'unique' => false])
            ->addIndex(['page_statut_id'], ['name' => "page_statut_id", 'unique' => false])
            ->addIndex(['alias_id'], ['name' => "alias_id", 'unique' => false])
            ->create();

        $this
            ->table("cms_page_boutique", ['id' => false, 'primary_key' => ["cms_page_boutique_id"], 'engine' => "InnoDB", 'encoding' => "utf8", 'collation' => "utf8_general_ci", 'comment' => ""])
            ->addColumn('cms_page_boutique_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'identity' => 'enable', 'signed' => false])
            ->addColumn('cms_page_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'cms_page_boutique_id', 'signed' => false])
            ->addColumn('boutique_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'cms_page_id', 'signed' => false])
            ->addColumn('cms_page_boutique_statut_id', 'integer', ['null' => false, 'default' => '1', 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'boutique_id'])
            ->addForeignKey('cms_page_id', 'cms_page', 'cms_page_id', array(
                'delete' => 'CASCADE',
                'update' => 'CASCADE'
            ))
            ->addForeignKey('boutique_id', 'boutique', 'boutique_id', array(
                'delete' => 'CASCADE',
                'update' => 'CASCADE'
            ))
            ->addIndex(['cms_page_id', 'boutique_id'], ['name' => "cms_page_id_2", 'unique' => true])
            ->addIndex(['cms_page_id'], ['name' => "cms_page_id", 'unique' => false])
            ->addIndex(['boutique_id'], ['name' => "boutique_id", 'unique' => false])
            ->addIndex(['cms_page_boutique_statut_id'], ['name' => "cms_page_boutique_statut_id", 'unique' => false])
            ->create();

        $this
            ->table("cms_page_nom", ['id' => false, 'primary_key' => ["cms_page_nom_id"], 'engine' => "InnoDB", 'encoding' => "utf8", 'collation' => "utf8_general_ci", 'comment' => ""])
            ->addColumn('cms_page_nom_id', 'integer', ['null' => false, 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'identity' => 'enable', 'signed' => false])
            ->addColumn('cms_page_id', 'integer', ['null' => false, 'default' => '0', 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'cms_page_nom_id', 'signed' => false])
            ->addColumn('langue_id', 'integer', ['null' => false, 'default' => '1', 'limit' => MysqlAdapter::INT_REGULAR, 'precision' => 10, 'after' => 'cms_page_id', 'signed' => false])
            ->addColumn('page_nom_nom', 'string', ['null' => false, 'default' => '', 'limit' => 255, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'langue_id'])
            ->addColumn('page_nom_image', 'string', ['null' => false, 'default' => '', 'limit' => 255, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'page_nom_nom'])
            ->addColumn('page_nom_url', 'string', ['null' => false, 'default' => '', 'limit' => 255, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'page_nom_image'])
            ->addColumn('page_nom_contenu', 'text', ['null' => false, 'limit' => MysqlAdapter::TEXT_LONG, 'collation' => "utf8_general_ci", 'encoding' => "utf8", 'after' => 'page_nom_url'])
            ->addForeignKey('cms_page_id', 'cms_page', 'cms_page_id', array(
                'delete' => 'CASCADE',
                'update' => 'CASCADE'
            ))
            ->addForeignKey('langue_id', 'langue', 'langue_id', array(
                'delete' => 'CASCADE',
                'update' => 'CASCADE'
            ))
            ->addIndex(['page_nom_url'], ['name' => "page_nom_url", 'unique' => true])
            ->addIndex(['cms_page_id'], ['name' => "cms_page_id", 'unique' => false])
            ->addIndex(['langue_id'], ['name' => "langue_id", 'unique' => false])
            ->addIndex(['cms_page_id', 'langue_id'], ['name' => "langue_id_2", 'unique' => true])
            ->create();
    }
}
