<?php

namespace Controller;

use Breadcrumb;
use Client;
use Panier;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Adresse;
use Livraison;
use Paiement;
use Texte;
use Tpe;
use Commande;
use DediException;
use Avoir;
use BonAchat;
use Pays;

use Symfony\Component\Routing\Annotation\Route;

/**
 * Class CoordonneesController
 * @package Controller
 */
class CoordonneesController extends Controller {

    /**
     * @Route(
     *      path         = "/checkout_shipping.html",
     *      name         = "coordonnees",
     *      methods      = { "GET" },
     *      defaults     = {"role" = "customer"}
     * )
     *
     * @return mixed
     */
    public function show()
    {
        $panier = Panier::getInstance();

        if($panier->isEmpty()){
            return $this->redirect($this->generateUrl('panier'));
        }

        $adresse = new Adresse();
        $livraison = new Livraison();
        $client = $this->getCurrentClient();

        list($sTotalHT, $sTotalTTC) = $panier->getTotalProducts();
        list($totalHT, $tolatTTC) = $panier->getAPayer();

        $this->template->assign('aPayerHT', $totalHT);
        $this->template->assign('aPayerTTC', $tolatTTC);

        $adresseDefault = $adresse->getDefault($client->client_id);

        $shipping = $panier->getShipping();

        // init addresse shipping
        if ($adresseDefault->isLoaded()) {
            $addrShip = $panier->getAddressShipping();
            if (!$addrShip instanceof Adresse || is_null($addrShip) || !$addrShip->isLoaded()) {
                $panier->setAddressShipping($adresseDefault);
            }
        }

        // init addresse billing
        $addrBill = $panier->getAddressBilling();
        if ($addrBill instanceof Adresse || is_null($addrBill) || !$addrBill->isLoaded()) {
            $addrBilling = $adresse->getBilling($client->client_id);
            if ($addrBilling->isLoaded()) {
                $panier->setAddressBilling($addrBilling);
            } elseif($adresseDefault->isLoaded()) {
                $panier->setAddressBilling($adresseDefault);
            }
        }

        $panierAdresses = array(
            'shipping' => $panier->getAddressShipping(),
            'billing' => $panier->getAddressBilling(),
        );
        $weight = $panier->getWeight();

        $paramLivr = array();
        $paramLivr['pays_id'] = $panierAdresses['shipping']->pays_id;
        $paramLivr['poids'] = $weight;
        $paramLivr['prix'] = $sTotalHT;

        if ($panier->isTva()) {
            $paramLivr['prix_ttc'] = $sTotalTTC;
        } else {
            $paramLivr['prix_ht'] = $sTotalHT;
        }
        $livraisons = $livraison->get($paramLivr);

        $this->template->assign('livraisons', $livraisons);
        $this->template->assign('port', !empty($shipping) ? $shipping->toArray() : array());

        $flashBag = $this->getFlashBag();
        if ($flashBag->has('error')) {
            $this->template->assign('error', $flashBag->get('error'));
        }
        if ($flashBag->has('success')) {
            $this->template->assign('success', $flashBag->get('success'));
        }

        // Envois des adresse au tpl
        // TODO Récupré l'adresse setter dans le panier pour qu'elle soit en selected dans le select
        $clientAdresses = $adresse->get(array(
            'client_id' => $client->client_id
        ));

        foreach ($clientAdresses as $i => &$clientAdresse) {
            if ($clientAdresse['adresse_id'] == $panierAdresses['shipping']->adresse_id) {
                $clientAdresse['selected'] = true;
            } else {
                $clientAdresse['selected'] = false;
            }
        }

        $this->template->assign('adresses', $clientAdresses);
        $this->template->assign('selectedAdresse', $panierAdresses['shipping']->toArray());

        return $this->render('delivery.tpl');
    }

    /**
     * @Route(
     *      path         = "/checkout_shipping.html",
     *      name         = "coordonnees_post",
     *      methods      = { "POST" },
     *      defaults     = {"role" = "customer"}
     * )
     *
     * @return mixed
     */
    public function post()
    {

        $flashBag = $this->getFlashBag();
        $texte = new Texte();
        $panier = Panier::getInstance();

        if ($this->request->get('livraison_id') == false) {
            $flashBag->add(
                'error',
                $texte->getOneOrCreate('errSelectLivraison', 'Vous devez selectionner un mode de livraison')
            );
        }

        $addrShipping = $panier->getAddressShipping();
        if (! $addrShipping instanceof Adresse || empty($addrShipping->adresse_id)) {
            $flashBag->add(
                'error',
                $texte->getOneOrCreate('errAddrLivraison', 'Vous devez selectionner une adresse de livraison')
            );
        }

        if ($flashBag->has('error') == false) {
            $url = $this->generateUrl('recapitulatif');
        } else {
            $url = $this->generateUrl('coordonnees');
        }

        return $this->redirect($url);
    }
}
