<?php

class Mailer extends Model
{

    protected $table = "email";

    function __construct(EmailSender $sender, $id = null)
    {
        parent::__construct($id);
        $this->sender = $sender;
    }

    public function get($formvars = array())
    {
        $params = array();
        if (!empty($formvars["email_id"]))
            $params["email_id"] = $formvars["email_id"];
        if (!empty($formvars["email_code"]))
            $params["email_code"] = $formvars["email_code"];
        if (isset($formvars["email_statut"]))
            $params["email_statut"] = $formvars["email_statut"];

        return parent::get($params);
    }

    /**
     * Used to send emails from database
     *
     * $to can be :
     * - array('john@doe.com' => 'John Doe')
     * - array('receiver@domain.org', 'other@domain.org' => 'A name')
     *
     * @param $sender
     * @param $email_code
     * @param $to
     * @param array $tags
     * @param array $opts
     * @param array $files
     * @throws Exception
     */
    public function send($email_code, $to, $tags = array(), $files = array())
    {
        // check si $email_code renseigné
        if (empty($email_code)) {
            throw new Exception('Mail code is missing');
        }

        // recup du mail
        $theMail = $this->get(array('email_code' => $email_code));

        if (empty($theMail)) {
            throw new Exception('No mail found with the code : "' . $email_code . "'");
        } elseif (empty($theMail[0]['email_statut'])) {
            throw new Exception('This mail is disable : "' . $email_code . "'");
        }

        $texte = new Texte();
        $subject = $texte->replaceTag($theMail[0]['email_nom_sujet'], $tags);
        $body = $texte->replaceTag($theMail[0]['email_nom_content'], $tags);

        if (!empty($tags)) {
            foreach ($tags as $key => $value) {
                $body = $texte->replaceIfTag(mb_strtoupper($key), $value, $body);
            }
        }

        $from = array(Boutique::getInstance()->contactEmail => Boutique::getInstance()->nom);
        $this->sender->send(EmailSender::createMessage($to, $subject, $body, $from, $files));
    }
}