<?php


namespace Shopbydedi\Payment;

use Panier;
use Tpe;
use Application;
use Client;
use Boutique;

/**
 * Class Systempay
 * @package Shopbydedi\Payment
 */
class Systempay implements PaymentInterface
{

    /**
     * @var \Router
     */
    protected $router;

    /**
     * @param $config
     *
     * @throws \DediException
     */
    public function __construct($config)
    {

        if (empty($config['systempay'])) {
            throw new \DediException('configure infos systempay in boutique config');
        }

        $this->version = 'V2';
        $this->key = $config['systempay']['key'];
        $this->ctx_mode = $config['systempay']['ctx_mode'];
        $this->site_id = $config['systempay']['site_id'];
        $this->capture_delay = '';
        $this->currency = '978';
        $this->payment_cards = 'CB;VISA;MASTERCARD;E-CARTEBLEUE;MAESTRO';
        $this->payment_config = 'SINGLE';
        $this->img_dir = '/media/paiement/systempay/';

        $this->router = Application::getRouter();
    }

    /**
     * @param Tpe $tpe
     * @param $amount
     *
     * @return array
     */
    public function bootRequest(Tpe $tpe, $amount)
    {
        // generation du numero de transaction
        date_default_timezone_set("UTC");
        $ts = time();
        $trans_date = date("YmdHis", $ts);
        $trans_id = date("His", $ts);

        // parametres bancaire
        $params = array();

        $params['vads_site_id'] = $this->site_id;
        $params['vads_ctx_mode'] = $this->ctx_mode;
        $params['vads_trans_id'] = $trans_id;
        $params['vads_trans_date'] = $trans_date;
        $params['vads_order_id'] = $tpe->tpe_reference_id;
        $params['vads_cust_id'] = $tpe->getClient()->client_id;
        $params['vads_cust_name'] = $tpe->getClient()->client_nom;
        $params['vads_cust_email'] = $tpe->getClient()->client_email;
        $params['vads_amount'] = round($amount * 100);
        $params['vads_currency'] = $this->currency;
        $params['vads_action_mode'] = 'INTERACTIVE';
        $params['vads_page_action'] = 'PAYMENT';
        $params['vads_version'] = $this->version;
        $params['vads_payment_config'] = $this->payment_config;
        $params['vads_capture_delay'] = 0;
        $params['vads_validation_mode'] = 0;
        $params['vads_url_return'] = substr(SITE_URL, 0, strlen(SITE_URL) - 1) . $this->router->getRouter()->generate('commande', ['_locale' => Boutique::getInstance()->getLangue()->ini]);


        return $params;
    }

    /**
     * @param Tpe $tpe
     * @param float $amount
     * @param array $params
     * @return mixed
     */
    public function request(Tpe $tpe, $amount, $params = array())
    {
        $params = array_merge($this->bootRequest($tpe, $amount), $params);

        $params['vads_url_success'] = substr(SITE_URL, 0, strlen(SITE_URL) - 1) . $this->router->getRouter()->generate('paiement_success', ['_locale' => Boutique::getInstance()->getLangue()->ini]);

        $arrCard = explode(";", $this->payment_cards);

        $out = "<div class='frm_systempay' style='text-align:center'>\n";

        foreach ($arrCard as $card) {
            $params['vads_payment_cards'] = $card;
            ksort($params);

            $signature_contents = "";
            foreach ($params as $key => $val) {
                if (strpos($key, 'vads_') !== false) {
                    if (!empty($signature_contents)) $signature_contents .= '+';
                    $signature_contents .= $val;
                }
            }
            $signature_contents .= '+' . $this->key;
            $params['signature'] = sha1($signature_contents);

            $out .= '<form method="POST" action="https://systempay.cyberpluspaiement.com/vads-payment/" style="display:inline;">' . "\n";
            foreach ($params as $key => $val) {
                $out .= '<input type="hidden" name="' . $key . '" value="' . $val . '" />' . "\n";
            }
            $out .= '<input type="image" name="payer" src="' . $this->img_dir . $card . '.png" />' . "\n";
            $out .= '</form>' . "\n";
        }

        $out .= '</div>';

        return $out;
    }

    /**
     * @param Panier $panier
     * @param float $amount
     * @param array $params
     * @return mixed
     */
    public function requestFromPanier(Panier $panier, $amount, $params = array())
    {
        $params = array_merge($this->bootRequest(new Tpe($panier->getReference()), $amount), $params);

        $params['vads_url_success'] = substr(SITE_URL, 0, strlen(SITE_URL) - 1) . $this->router->getRouter()->generate('commande', ['_locale' => Boutique::getInstance()->getLangue()->ini]);

        $arrCard = explode(";", $this->payment_cards);
        $out = "<div class='frm_systempay' style='text-align:center'>\n";

        foreach ($arrCard as $card) {
            $params['vads_payment_cards'] = $card;
            ksort($params);

            $signature_contents = "";
            foreach ($params as $key => $val) {
                if (strpos($key, 'vads_') !== false) {
                    if (!empty($signature_contents)) $signature_contents .= '+';
                    $signature_contents .= $val;
                }
            }
            $signature_contents .= '+' . $this->key;
            $params['signature'] = sha1($signature_contents);

            $out .= '<form method="POST" action="https://systempay.cyberpluspaiement.com/vads-payment/" style="display:inline;">' . "\n";
            foreach ($params as $key => $val) {
                $out .= '<input type="hidden" name="' . $key . '" value="' . $val . '" />' . "\n";
            }
            $out .= '<input type="image" name="payer" src="' . $this->img_dir . $card . '.png" />' . "\n";
            $out .= '</form>' . "\n";
        }

        $out .= '</div>';

        return $out;
    }

    /**
     * @param array $request
     * @return mixed
     */
    public function response($request)
    {
        if (empty($request)) return;

        ksort($request);

        $signature_contents = "";
        foreach ($request as $key => $val) {
            if (strpos($key, 'vads_') !== false) {
                if (!empty($signature_contents)) $signature_contents .= '+';
                $signature_contents .= $val;
            }
        }

        $signature_contents .= '+' . $this->key;
        $signature = sha1($signature_contents);

        // on verifie si la signature est ok
        if ($signature != $request['signature']) return;

        $order_info = sha1($request['vads_order_id'] . "+" . $request['vads_trans_id'] . "+" . $request['vads_amount'] . "+" . $request['vads_site_id'] . "+" . $this->key); // pour securiser un peu plus

        // on verifie si la signature renforce est ok
        if ($order_info != $request['vads_order_info']) return;

        $tpe = new Tpe;
        if ($request['vads_result'] == "00") {
            $tpe->paiementOK($request['vads_order_id'], $request);
        } else {
            $tpe->paiementKO($request['vads_order_id'], $request);
        }
    }
}