<?php

/**
 * Class Stock
 */
class Stock extends Model
{

    protected $table = 'produit_stock';

    /**
     * gestion centralisee du mouvement de stock
     *
     * @param $idProduit
     * @param string $idAttribut
     * @param string $idCommande
     * @param string $idFournisseur
     * @param int $qte
     * @param int $prixAchat
     * @param string $mvtInfo
     * @param string $mvtPhys
     * @param string $commentaire
     * @param null $stockType
     * @param null $fcp_id
     * @param null $accise
     *
     * @return bool
     */
    function mouvement($idProduit, $idAttribut = "NULL", $idCommande = "NULL", $idFournisseur = "NULL", $qte = 0, $prixAchat = 0, $mvtInfo = "", $mvtPhys = "", $commentaire = "", $stockType = null, $fcp_id = null, $accise = null)
    {

        if (empty($idProduit)) {
            return false;
        }

        if (empty($mvtInfo) && empty($mvtPhys))
            return false;

        if (empty($idAttribut))
            $idAttribut = 'NULL';
        if (empty($idCommande))
            $idCommande = 'NULL';
        if (empty($idFournisseur))
            $idFournisseur = 'NULL';

        // on verifie si la qte pass�e est bien format�
        if (!is_numeric($qte)) {
            if (empty($qte))
                $qte = 0;
            else
                return false;
        }

        // mouvement informatique
        if (!empty($mvtInfo)) {
            if ($mvtInfo == 'out')
                $newQte = $qte * -1;
            else
                $newQte = $qte;

            if (!empty($stockType)) {
                if ($idAttribut != 'NULL') {
                    $sql = "SELECT * FROM produit_attribut_stock";
                    $sql .= " WHERE produit_attribut_id = '" . $idAttribut . "'";
                    $sql .= " AND stock_type_id = '" . $stockType . "'";

                    $result = $this->sql->getAll($sql);

                    $keyword = (!empty($result)) ? "UPDATE" : "INSERT INTO";
                    $sql = $keyword . " produit_attribut_stock SET";
                    $sql .= " produit_attribut_stock_qte = produit_attribut_stock_qte + " . $newQte;
                    if (!empty($result)) {
                        $sql .= " WHERE produit_attribut_id = '" . $idAttribut . "'";
                        $sql .= " AND stock_type_id = '" . $stockType . "'";
                    } else {
                        $sql .= ", produit_attribut_id = '" . $idAttribut . "'";
                        $sql .= ", stock_type_id = '" . $stockType . "'";
                    }
                    $this->sql->query($sql);
                }
            } else {
                if ($idAttribut != 'NULL') {
                    $sql = "UPDATE produit_attribut SET produit_attribut_quantite = produit_attribut_quantite + " . $newQte . " WHERE produit_attribut_id = '" . $idAttribut . "'";
                    $this->sql->query($sql);
                }
            }
        }

        // mouvement physique
        if (!empty($mvtPhys)) {

            if ($mvtPhys == 'out')
                $newQte = $qte * -1;
            else
                $newQte = $qte;

            if (empty($stockType)) {
                if ($idAttribut != 'NULL') {
                    $sql = "UPDATE produit_attribut SET produit_attribut_quantite_p = produit_attribut_quantite_p + " . $newQte . " WHERE produit_attribut_id = '" . $idAttribut . "'";
                    $this->sql->query($sql);
                }
            }

            if (empty($commentaire)) {
                if (!empty($idCommande) && $idCommande != 'NULL') {
                    if ($mvtPhys == 'out')
                        $commentaire = "Sortie Commande " . $idCommande;
                    else
                        $commentaire = "Entree Commande " . $idCommande;
                } elseif (!empty($idFournisseur) && $idFournisseur != 'NULL') {
                    if ($mvtPhys == 'out')
                        $commentaire = "Sortie Fournisseur " . $idCommande;
                    else
                        $commentaire = "Entree Fournisseur " . $idCommande;
                }
            }

            Tools::escapeDataToSql($commentaire);

            $sql = "INSERT INTO produit_stock SET";
            $sql .= " produit_id = '" . $idProduit . "'";
            $sql .= ", produit_attribut_id = " . $idAttribut;
            $sql .= ", fournisseur_id = " . $idFournisseur;
            $sql .= ", commande_id = " . $idCommande;
            $sql .= ", produit_stock_quantite = " . $newQte;
            $sql .= ", produit_stock_libelle = '" . $commentaire . "'";
            $sql .= ", produit_stock_prix_ht = '" . $prixAchat . "'";
            $sql .= ", produit_stock_date_ajout = NOW()";
            $sql .= ", produit_stock_date_modif = NOW()";
            if (!empty($stockType)) {
                $sql .= ", stock_type_id = '" . $stockType . "'";
            }
            if (!empty($fcp_id)) {
                $sql .= ", fournisseur_commande_produit_id = '" . $fcp_id . "'";
            }

            $this->sql->query($sql);
        }
    }

    /**
     * @param $pId
     * @param $qte
     * @param $achat
     * @param null $attId
     *
     * @return float
     *
     * @throws DediException
     * @throws Exception
     */
    public function updatePump($pId, $qte, $achat, $attId = null)
    {
        $query = new Query();
        if (is_null($attId)) {
            $query->reset()->select()->from('produit')->where('produit_id', $pId);
            $result = $this->sql->getOne($query->build());

            if (empty($result)) {
                throw new DediException('no product for id ' . $pId);
            }

            $pump = (($result['produit_quantite_p'] * $result['produit_achat']) + ($qte * $achat)) / ($result['produit_quantite_p'] + $qte);

            $query->reset()->update('produit')
                ->value('produit_achat', $pump)
                ->value('produit_achat_der', $achat)
                ->where('produit_id', $pId);

            $this->sql->query($query->build());
        } else {
            $query->reset()->select()->from('produit_attribut')
                ->where('produit_id', $pId)
                ->where('produit_attribut_id', $attId);
            $result = $this->sql->getOne($query->build());

            if (empty($result)) {
                throw new DediException('no att for id ' . $attId);
            }

            $pump = (($result['produit_attribut_quantite_p'] * $result['produit_attribut_achat']) + ($qte * $achat)) / ($result['produit_attribut_quantite_p'] + $qte);

            $query->reset()->update('produit_attribut')
                ->value('produit_attribut_achat', $pump)
                ->value('produit_attribut_achat_der', $achat)
                ->where('produit_attribut_id', $attId);

            $this->sql->query($query->build());
        }

        return $pump;
    }

    /**
     * @param $produitId
     */
    function updateProdWithAtt($produitId)
    {

        // on recupere la somme des quantites
        $sql = "SELECT SUM(produit_attribut_quantite) as qte_info, SUM(produit_attribut_quantite_p) as qte_phys";
        $sql .= " FROM (produit_attribut)";
        $sql .= " WHERE produit_id = '" . $produitId . "'";
        $result = $this->sql->getOne($sql);
        $qteInfo = $result['qte_info'];
        $qtePhys = $result['qte_phys'];

        // on met a jour les qtes
        $sql = "UPDATE produit SET";
        $sql .= " produit_quantite = '" . $qteInfo . "'";
        $sql .= ", produit_quantite_p = '" . $qtePhys . "'";
        //$sql.=", produit_achat = '0'";
        $sql .= " WHERE produit_id = '" . $produitId . "'";
        $this->sql->query($sql);

        // on vide les infos stocks si besoin
        $sql = "DELETE FROM produit_stock WHERE produit_id = '" . $produitId . "' AND produit_attribut_id IS NULL";
        $this->sql->query($sql);
    }

    function checkStock()
    {

        $arrStock = $this->initMvt();
        $listStatut = array();
        foreach ($arrStock[0] as $key => $value) {
            if ($value[0] == 'out' && $value[1] == '') {
                array_push($listStatut, $key);
            }
        }

        $sql = "SELECT p.produit_id, p.produit_quantite, p.produit_quantite_p, p.produit_ref, pn.produit_nom_nom";
        $sql .= " FROM produit p, produit_nom pn";
        $sql .= " WHERE p.produit_statut_id != 3";
        $sql .= " AND p.produit_id = pn.produit_id";
        $sql .= " AND pn.langue_id = '1'";
        $arrProd = $this->sql->getAll($sql);

        foreach ($arrProd as $p) {

            $sql = "SELECT pa.produit_attribut_id, pa.produit_attribut_quantite, pa.produit_attribut_quantite_p, pa.produit_attribut_ref, povn.produit_option_valeur_nom";
            $sql .= " FROM produit_attribut pa, produit_option_valeur_nom povn";
            $sql .= " WHERE pa.produit_id = '" . $p['produit_id'] . "'";
            $sql .= " AND pa.produit_option_valeur_id = povn.produit_option_valeur_id";
            $sql .= " AND povn.langue_id = '1'";
            $arrAtt = $this->sql->getAll($sql);

            if (!empty($arrAtt)) {

                foreach ($arrAtt as $att) {
                    // recuperation du nomnbre de produit en attente de livraison
                    $sql = "SELECT SUM(produit_option_valeur_quantite) as qte";
                    $sql .= " FROM commande_produit_attribut cpa, commande_produit cp, commande c";
                    $sql .= " WHERE cpa.produit_attribut_id = '" . $att['produit_attribut_id'] . "'";
                    $sql .= " AND cpa.commande_produit_id = cp.commande_produit_id";
                    $sql .= " AND cp.commande_id = c.commande_id";
                    $sql .= " AND c.commande_statut_id IN (" . implode(",", $listStatut) . ")";
                    $result = $this->sql->getOne($sql);
                    $qteEnCours = $result['qte'];
                    if (empty($qteEnCours))
                        $qteEnCours = 0;
                    $qteInfo = $att['produit_attribut_quantite'] + $qteEnCours;
                    $qteStock = $att['produit_attribut_quantite_p'];
                    if ($qteInfo != $qteStock) {
                        echo '<b>Erreur de stock</b><br/>';
                        echo 'Produit ' . $p['produit_id'] . ' / ' . $p['produit_ref'] . ' / ' . $p['produit_nom_nom'] . '<br/>';
                        echo 'Attribut ' . $att['produit_attribut_id'] . ' / ' . $att['produit_attribut_ref'] . ' / ' . $att['produit_option_valeur_nom'] . '<br/>';
                        echo 'qteInfo = ' . $qteInfo . ' (dont ' . $qteEnCours . ' encours) / qteStock = ' . $qteStock . '<br/><br/>';
                    }
                }
            }
        }
    }

    // tableau permettant la mise a jour des stocks, indispensable
    function initMvt()
    {

        // cas ou le stock informatique est modifie au moment du paiement
        /*$arrMvtStatut = array(
            '0' => array(
                Commande::STATUT_ATTENTE => array('', ''),
                Commande::STATUT_PAIEMENT => array('out', ''),
                Commande::STATUT_PREPARATION => array('out', ''),
                Commande::STATUT_EXPEDIE => array('out', 'out'),
                Commande::STATUT_RELANCE => array('', ''),
                Commande::STATUT_ANNULEE => array('', ''),
                '0' => array('', '')
            ),
            Commande::STATUT_ATTENTE => array(
                Commande::STATUT_ATTENTE => array('', ''),
                Commande::STATUT_PAIEMENT => array('out', ''),
                Commande::STATUT_PREPARATION => array('out', ''),
                Commande::STATUT_EXPEDIE => array('out', 'out'),
                Commande::STATUT_RELANCE => array('', ''),
                Commande::STATUT_ANNULEE => array('', ''),
                '0' => array('', '')
            ),
            Commande::STATUT_PAIEMENT => array(
                Commande::STATUT_ATTENTE => array('in', ''),
                Commande::STATUT_PAIEMENT => array('', ''),
                Commande::STATUT_PREPARATION => array('', ''),
                Commande::STATUT_EXPEDIE => array('', 'out'),
                Commande::STATUT_RELANCE => array('in', ''),
                Commande::STATUT_ANNULEE => array('in', ''),
                '0' => array('in', '')
            ),
            Commande::STATUT_PREPARATION => array(
                Commande::STATUT_ATTENTE => array('in', ''),
                Commande::STATUT_PAIEMENT => array('', ''),
                Commande::STATUT_PREPARATION => array('', ''),
                Commande::STATUT_EXPEDIE => array('', 'out'),
                Commande::STATUT_RELANCE => array('in', ''),
                Commande::STATUT_ANNULEE => array('in', ''),
                '0' => array('in', '')
            ),
            Commande::STATUT_EXPEDIE => array(
                Commande::STATUT_ATTENTE => array('in', 'in'),
                Commande::STATUT_PAIEMENT => array('', 'in'),
                Commande::STATUT_PREPARATION => array('', 'in'),
                Commande::STATUT_EXPEDIE => array('', ''),
                Commande::STATUT_RELANCE => array('in', 'in'),
                Commande::STATUT_ANNULEE => array('in', 'in'),
                '0' => array('in', 'in')
            ),
            Commande::STATUT_RELANCE => array(
                Commande::STATUT_ATTENTE => array('', ''),
                Commande::STATUT_PAIEMENT => array('out', ''),
                Commande::STATUT_PREPARATION => array('out', ''),
                Commande::STATUT_EXPEDIE => array('out', 'out'),
                Commande::STATUT_RELANCE => array('', ''),
                Commande::STATUT_ANNULEE => array('', ''),
                '0' => array('', '')
            ),
            Commande::STATUT_ANNULEE => array(
                Commande::STATUT_ATTENTE => array('', ''),
                Commande::STATUT_PAIEMENT => array('out', ''),
                Commande::STATUT_PREPARATION => array('out', ''),
                Commande::STATUT_EXPEDIE => array('out', 'out'),
                Commande::STATUT_RELANCE => array('', ''),
                Commande::STATUT_ANNULEE => array('', ''),
                '0' => array('', '')
            )
        );*/


        // cas ou le stock informatique est modifie a la creation de la commande
        $arrMvtStatut = array(
            '0' => array(
                Commande::STATUT_ATTENTE => array('out', ''),
                Commande::STATUT_PAIEMENT => array('out', ''),
                Commande::STATUT_PREPARATION => array('out', ''),
                Commande::STATUT_EXPEDIE => array('out', 'out'),
                Commande::STATUT_RELANCE => array('out', ''),
                Commande::STATUT_ANNULEE => array('', ''),
                '0' => array('', '')
            ),
            Commande::STATUT_ATTENTE => array(
                Commande::STATUT_ATTENTE => array('', ''),
                Commande::STATUT_PAIEMENT => array('', ''),
                Commande::STATUT_PREPARATION => array('', ''),
                Commande::STATUT_EXPEDIE => array('', 'out'),
                Commande::STATUT_RELANCE => array('', ''),
                Commande::STATUT_ANNULEE => array('in', ''),
                '0' => array('in', '')
            ),
            Commande::STATUT_PAIEMENT => array(
                Commande::STATUT_ATTENTE => array('', ''),
                Commande::STATUT_PAIEMENT => array('', ''),
                Commande::STATUT_PREPARATION => array('', ''),
                Commande::STATUT_EXPEDIE => array('', 'out'),
                Commande::STATUT_RELANCE => array('', ''),
                Commande::STATUT_ANNULEE => array('in', ''),
                '0' => array('in', '')
            ),
            Commande::STATUT_PREPARATION => array(
                Commande::STATUT_ATTENTE => array('', ''),
                Commande::STATUT_PAIEMENT => array('', ''),
                Commande::STATUT_PREPARATION => array('', ''),
                Commande::STATUT_EXPEDIE => array('', 'out'),
                Commande::STATUT_RELANCE => array('', ''),
                Commande::STATUT_ANNULEE => array('in', ''),
                '0' => array('in', '')
            ),
            Commande::STATUT_EXPEDIE => array(
                Commande::STATUT_ATTENTE => array('', 'in'),
                Commande::STATUT_PAIEMENT => array('', 'in'),
                Commande::STATUT_PREPARATION => array('', 'in'),
                Commande::STATUT_EXPEDIE => array('', ''),
                Commande::STATUT_RELANCE => array('', 'in'),
                Commande::STATUT_ANNULEE => array('in', 'in'),
                '0' => array('in', 'in')
            ),
            Commande::STATUT_RELANCE => array(
                Commande::STATUT_ATTENTE => array('', ''),
                Commande::STATUT_PAIEMENT => array('', ''),
                Commande::STATUT_PREPARATION => array('', ''),
                Commande::STATUT_EXPEDIE => array('', 'out'),
                Commande::STATUT_RELANCE => array('', ''),
                Commande::STATUT_ANNULEE => array('in', ''),
                '0' => array('in', '')
            ),
            Commande::STATUT_ANNULEE => array(
                Commande::STATUT_ATTENTE => array('out', ''),
                Commande::STATUT_PAIEMENT => array('out', ''),
                Commande::STATUT_PREPARATION => array('out', ''),
                Commande::STATUT_EXPEDIE => array('out', 'out'),
                Commande::STATUT_RELANCE => array('out', ''),
                Commande::STATUT_ANNULEE => array('', ''),
                '0' => array('', '')
            )
        );

        return $arrMvtStatut;
    }

}
